/*******************************************************************************
  Clock System Service Interface Definition

  Company:
    Microchip Technology Inc.

  File Name:
    sys_clk.h

  Summary:
    API for the Clock System Service.

  Description:
    This file contains the interface definition for the Clock System
    Service.  It provides a way to interact with the Clock subsystem to
    manage the timing requests supported by the system.
*******************************************************************************/

//DOM-IGNORE-BEGIN
/*******************************************************************************
Copyright (c) 2013 released Microchip Technology Inc.  All rights reserved.

Microchip licenses to you the right to use, modify, copy and distribute
Software only when embedded on a Microchip microcontroller or digital signal
controller that is integrated into your product or third party product
(pursuant to the sublicense terms in the accompanying license agreement).

You should refer to the license agreement accompanying this Software for
additional information regarding your rights and obligations.

SOFTWARE AND DOCUMENTATION ARE PROVIDED AS IS WITHOUT WARRANTY OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE.
IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER
CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR
OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR
CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF
SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
(INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.
*******************************************************************************/
//DOM-IGNORE-END

#ifndef _SYS_CLK_H
#define _SYS_CLK_H


// *****************************************************************************
// *****************************************************************************
// Section: Included Files
// *****************************************************************************
// *****************************************************************************

#include "system_config.h"


// *****************************************************************************
// *****************************************************************************
// Section: SYS Clock Data Types
// *****************************************************************************
// *****************************************************************************

// *****************************************************************************
/*  Clocks enumeration

  Summary:
	Lists the available clock outputs.

  Description:
	This enumeration lists all of the available clock outputs. This is used 
        by the SYS_CLK_ClockFrequencyGet function.

  Remarks:
    'Clock output' means output from the clock source.
*/

typedef enum
{
    /* Selects System clock output. This is used by the
    CPU and some of the peripherals */
    CLK_SYSTEM /*DOM-IGNORE-BEGIN*/  = 0 /*DOM-IGNORE-END*/,

    /* Selects peripheral clock output. This is used by most
    of the peripherals */
    CLK_PERIPHERAL /*DOM-IGNORE-BEGIN*/  = 1 /*DOM-IGNORE-END*/,

    /* Selects USB clock output. This is used by the
    USB module */
    CLK_USB /*DOM-IGNORE-BEGIN*/  = 2 /*DOM-IGNORE-END*/,

    /* Selects referece clock output. This to check any of the
    clock or it could be used as a clock source for other devices */
    CLK_REFERENCE /*DOM-IGNORE-BEGIN*/  = 3 /*DOM-IGNORE-END*/,

    /* Selects Graphics clock output. This is used by the display
    module */
    CLK_GRAPHICS /*DOM-IGNORE-BEGIN*/  = 4 /*DOM-IGNORE-END*/,

    /* Selects ADC/PWM clock */
    CLK_ADCPWM /*DOM-IGNORE-BEGIN*/  = 5 /*DOM-IGNORE-END*/,

}SYS_CLK_OUTPUT;


// *****************************************************************************
// *****************************************************************************
// Section: SYS Clock Status Routines
// *****************************************************************************
// *****************************************************************************

//******************************************************************************
/* Function:
    unsigned long SYS_CLK_ClockFrequencyGet ( SYS_ CLK_OUTPUT clockOutput )

  Summary:
    Gets the selected clock frequency in Hz.

  Description:
    This function gets the selected clock frequency in Hz.

    system.h should have included this file.

    To override this function, in the system_config.h have:
        #define SYS_RESET_SoftwareReset() MyReset()

    Implement the function MyReset in system.c and provide a prototype in
    in system.h.

    In system.h
        void MyReset(void);

    In system.c
        void MyReset(void)
        {
            // User defined reset behavior
        }

  Precondition:
    The selected clock output must be configured and enabled.

  Parameters:
    Selected clock frequency in Hz.

  Returns:
    Clock frequency in Hz.

  Example:
    <code>
    unsigned long sysClockOutputHz;

    sysClockOutputHz = SYS_CLK_ClockFrequencyGet(CLK_SYSTEM);
    </code>

  Remarks:
 */

unsigned long SYS_CLK_ClockFrequencyGet ( SYS_CLK_OUTPUT clockOutput );

#endif //SYS_CLK_H

/*******************************************************************************
 End of File
*/

