/*******************************************************************************
  Device System Service Interface Definition

  Company:
    Microchip Technology Inc.

  File Name:
    sys_devcon.h

  Summary:
    Reset System Service interface definition.

  Description:
    This file contains the interface definition for the Device System
    Service.  It provides a way to interact with the device specific features
*******************************************************************************/

//DOM-IGNORE-BEGIN
/*******************************************************************************
Copyright (c) 2013 released Microchip Technology Inc.  All rights reserved.

Microchip licenses to you the right to use, modify, copy and distribute
Software only when embedded on a Microchip microcontroller or digital signal
controller that is integrated into your product or third party product
(pursuant to the sublicense terms in the accompanying license agreement).

You should refer to the license agreement accompanying this Software for
additional information regarding your rights and obligations.

SOFTWARE AND DOCUMENTATION ARE PROVIDED AS IS WITHOUT WARRANTY OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE.
IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER
CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR
OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR
CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF
SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
(INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.
*******************************************************************************/
//DOM-IGNORE-END

#ifndef _SYS_DEVCON_H
#define _SYS_DEVCON_H


// *****************************************************************************
// *****************************************************************************
// Section: Included Files
// *****************************************************************************
// *****************************************************************************

#include "system_config.h"

// *****************************************************************************
// *****************************************************************************
// Section: SYS RESET Module Routines
// *****************************************************************************
// *****************************************************************************

// *****************************************************************************
/* Function:
    void Nop( void )

  Summary:
    Generates a nop instruction

  Description:
    This function generates a nop instruction. It defaults to the compiler
    defaults providing a common interface across all compilers.

  PreCondition:
    None.

  Parameters:
    None.
      
  Returns:
    None.

*/

#if !defined(Nop) && defined(__XC16__)
        #define Nop()        __builtin_nop(void)
#elif !defined(NOP) && defined(__XC8__)
        #define Nop()        _nop(void)
#elif !defined(Nop) && defined(__XC32__)
        #define Nop()        _nop(void)
#else
        #define Nop()
#endif

// *****************************************************************************
/* Function:
    void SYS_WDT_TimerClear ( void )

  Summary:
    Reset the WDT.

  Description:
    This function clears the WDT counter. The WDT should be cleared periodically
    before the count overflows and forces the device to Reset.Clearing the WDT
    before the count reaches the window, will cause a reset in Windowed mode.

    The example code doesn't include the settings that should be done through
    Configuration bits.

  Precondition:
    None.

  Parameters:
    None.

  Returns:
    None.

  Example:
    <code>

    //Application loop
    while(1)
    {
        SYS_WDT_TimerClear();
        //user code
    }
    </code>

*/

#ifndef SYS_WDT_TimerClear
    #if defined(__XC8__)
        #define SYS_WDT_TimerClear()        asm("clrwdt")
    #elif defined(__XC16__)
        #define SYS_WDT_TimerClear()        {__asm__ volatile ("clrwdt");}
    #elif defined(__XC32__)
        #define SYS_WDT_TimerClear()        (WDTCONSET = _WDTCON_WDTCLR_MASK)
    #else
        #define SYS_WDT_TimerClear()
    #endif
#endif


// *****************************************************************************
/* Function:
    void Sleep( void )

  Summary:
    Puts the device into sleep

  Description:
    This function puts the device into sleep. The implementation is family
    specific.

    system.h should have included this file.

    To override this function, in the system_config.h have:
        #define Sleep() MySleep()

    Implement the function MySleep in system.c and provide a prototype in
    in system.h.

    In system.h
        void MySleep(void);

    In system.c
        void MySleep(void)
        {
            // User defined method to put the device in sleep.
        }


  PreCondition:
    None.

  Parameters:
    None.

  Returns:
    None.

*/

#if !defined(Sleep) && defined(__XC16__)
        #define Sleep()        {__asm__ volatile ("pwrsav #0");}
#elif !defined(SLEEP) && defined(__XC8__)
        #define Sleep()        asm("sleep")
#elif !defined(Sleep) && defined(__XC32__)
        #define Sleep()        
#else
        #define Sleep()
#endif

// *****************************************************************************
/* Function:
    void Idle( void )

  Summary:
    Puts the device into idle mode

  Description:
    This function puts the device into idle mode. The implementation is family
    specific.

    system.h should have included this file.

    To override this function, in the system_config.h have:
        #define Idle() MyIdle()

    Implement the function MyIdle in system.c and provide a prototype in
    in system.h.

    In system.h
        void MyIdle(void);

    In system.c
        void MyIdle(void)
        {
            // User defined method to put the device in idle mode.
        }


  PreCondition:
    None.

  Parameters:
    None.

  Returns:
    None.

*/

#if !defined(Idle) && defined(__XC16__)
        #define Idle()        {__asm__ volatile ("pwrsav #0");}
#elif !defined(Idle) && defined(__XC8__)
        #define Idle()
#elif !defined(Idle) && defined(__XC32__)
        #define Idle()
#else
        #define Idle()
#endif

#endif //_SYS_DEVCON_H

/*******************************************************************************
 End of File
*/

