/**
 * mikroc_i2c_hal.c
 *
 * Version : 1.0.1
 *
 * Revision Log :
 * 0.0.1 (Sep/2015) Module created                          Richard Lowe
 * 0.0.2 (Sep/2016) Documentation added                     Milos Vidojevic
 * 1.0.0 (Dec/2016) Major Updates                           Milos Vidojevic
 *                 - Static I2C address member removed from the module
 *                - Transfer added for devices with 16bit or greater register size 
 *                - Pin definitions removed for HAL layer. 
 *                - Delay ms function added
 * 1.0.1 (Jan/2017) dsPIC Ponters added                     Milos Vidojevic
 *
 * To Do List :
 *                 - GPIO HAL needed
 *                 - Delay us
 * 
 ******************************************************************************/


#include "winc1500_api.h"
#include <string.h>

/*                 Preprocessors
 ******************************************************************************/

#define READ_BIT                1
#define WRITE_BIT               0

#define MAX_RET(x,y) ((x)>=(y) ? (x) : (y))

#if defined( __MIKROC_PRO_FOR_PIC32__ ) || \
    defined( __MIKROC_PRO_FOR_DSPIC__ )
static uint8_t NACK_BIT   = 0x01;
static uint8_t ACK_BIT    = 0x00;
#elif defined( __MIKROC_PRO_FOR_PIC__ ) || \
      defined( __MIKROC_PRO_FOR_AVR__ )
static uint8_t NACK_BIT   = 0x00;
static uint8_t ACK_BIT    = 0x01;
#endif

/*      GPIO pins
 ******************************************************************************/

extern sfr sbit WIFI_7_RST;
extern sfr sbit WIFI_7_CS;
extern sfr sbit WIFI_7_EN;

static uint32_t _millis;

/*                I2C Pointers
 ******************************************************************************/

#if defined( __MIKROC_PRO_FOR_ARM__ )
    #if defined( __STM32__ ) || defined( __KINETIS__ ) || defined( __MCHP__ )
static unsigned int ( *start_i2c_p )            ( void );
static unsigned int ( *write_i2c_p )            ( unsigned char slave_address,
                                                  unsigned char *buffer,
                                                  unsigned long count,
                                                  unsigned long end_mode );
static void         ( *read_i2c_p )             ( unsigned char slave_address,
                                                  unsigned char *buffer,
                                                  unsigned long count,
                                                  unsigned long end_mode );
    #elif defined( __TI__ )
static void         ( *enable_i2c_p )           ( void );
static void         ( *disable_i2c_p )          ( void );
static void         ( *set_slave_address_i2c_p )( char slave_address,
                                                  char dir );
static unsigned char( *write_i2c_p )            ( unsigned char data_in,
                                                  unsigned char mode );
static unsigned char( *read_i2c_p )             ( unsigned char *data_out,
                                                  unsigned char mode );
    #endif
#elif  defined( __MIKROC_PRO_FOR_AVR__ )
static unsigned char( *busy_i2c_p )             ( void );
static unsigned char( *status_i2c_p )           ( void );
static unsigned char( *start_i2c_p )            ( void );
static void         ( *stop_i2c_p )             ( void );
static void         ( *close_i2c_p )            ( void );
static void         ( *write_i2c_p )            ( unsigned char data_out );
static unsigned char( *read_i2c_p )             ( unsigned char ack );

#elif  defined( __MIKROC_PRO_FOR_PIC__ )
static unsigned char( *is_idle_i2c_p )          ( void );
static unsigned char( *start_i2c_p )            ( void );
static void         ( *stop_i2c_p )             ( void );
static void         ( *restart_i2c_p )          ( void );
static unsigned char( *write_i2c_p )            ( unsigned char data_out );
static unsigned char( *read_i2c_p )             ( unsigned char ack );

#elif defined( __MIKROC_PRO_FOR_PIC32__ )
static unsigned int ( *is_idle_i2c_p )          ( void );
static unsigned int ( *start_i2c_p )            ( void );
static void         ( *stop_i2c_p )             ( void );
static unsigned int ( *restart_i2c_p )          ( void );
static unsigned int ( *write_i2c_p )            ( unsigned char data_out );
static unsigned char( *read_i2c_p )             ( unsigned int ack );

#elif defined( __MIKROC_PRO_FOR_DSPIC__ )
static unsigned int ( *is_idle_i2c_p )          ( void );
static unsigned int ( *start_i2c_p )            ( void );
static void         ( *stop_i2c_p )             ( void );
static void         ( *restart_i2c_p )          ( void );
static unsigned int ( *write_i2c_p )            ( unsigned char data_out );
static unsigned char( *read_i2c_p )             ( unsigned int ack );

#elif defined( __MIKROC_PRO_FOR_8051__ )
static unsigned char( *status_i2c_p )           ( void );
static unsigned char( *start_i2c_p )            ( void );
static void         ( *stop_i2c_p )             ( void );
static void         ( *close_i2c_p )            ( void );
static void         ( *write_i2c_p )            ( unsigned char data_out );
static unsigned char( *read_i2c_p )             ( unsigned char ack );

#elif defined( __MIKROC_PRO_FOR_FT90x__ )
static void         ( *soft_reset_i2c_p )       ( void );
static void         ( *set_slave_address_i2c_p )( unsigned char slave_address );
static unsigned char( *write_i2c_p )            ( unsigned char *buffer,
                                                  unsigned int count );
static unsigned char( *read_i2c_p )             ( unsigned char *buffer,
                                                  unsigned int count );
#endif

/*                SPI Pointers
 ******************************************************************************/

#if defined( __MIKROC_PRO_FOR_ARM__ )   || \
    defined( __MIKROC_PRO_FOR_DSPIC__ )
static void         ( *write_spi_p )            ( unsigned int data_out );
static unsigned int ( *read_spi_p )             ( unsigned int buffer );

#elif defined( __MIKROC_PRO_FOR_AVR__ )     || \
      defined( __MIKROC_PRO_FOR_PIC__ )     || \
      defined( __MIKROC_PRO_FOR_8051__ )
static void         ( *write_spi_p )            ( unsigned char data_out );
static unsigned char( *read_spi_p )             ( unsigned char dummy );

#elif defined( __MIKROC_PRO_FOR_PIC32__ )
static void         ( *write_spi_p )            ( unsigned long data_out );
static unsigned long( *read_spi_p )             ( unsigned long dummy );

#elif defined( __MIKROC_PRO_FOR_FT90x__ )
static void         ( *write_spi_p )            ( unsigned char data_out );
static unsigned char( *read_spi_p )             ( unsigned char dummy );
#endif

/*                UART Pointers
 ******************************************************************************/

#if defined( __MIKROC_PRO_FOR_ARM__ )       || \
    defined( __MIKROC_PRO_FOR_DSPIC__ )     || \
    defined( __MIKROC_PRO_FOR_PIC32__ )
static void         ( *write_uart_p )           ( unsigned int data_out );

#elif defined( __MIKROC_PRO_FOR_AVR__ )     || \
      defined( __MIKROC_PRO_FOR_PIC__ )     || \
      defined( __MIKROC_PRO_FOR_8051__ )    || \
      defined( __MIKROC_PRO_FOR_FT90x__ )
static void         ( *write_uart_p )           ( unsigned char data_out );
#endif


/*                Public Function Definitions
 ******************************************************************************/

void hal_wifi7_delay( uint32_t ms )
{
#if defined(__MIKROC_PRO_FOR_ARM__)     || \
    defined( __MIKROC_PRO_FOR_FT90x__ ) || \
    defined( __MIKROC_PRO_FOR_AVR__ )   || \
    defined( __MIKROC_PRO_FOR_8051__ )  || \
    defined( __MIKROC_PRO_FOR_DSPIC__ ) || \
    defined( __MIKROC_PRO_FOR_PIC32__ ) || \
    defined( __MIKROC_PRO_FOR_PIC__ )
    while( ms-- )
        Delay_1ms();
#endif
}

void hal_wifi7_init( void )
{
#if defined( __MIKROC_PRO_FOR_ARM__ )
    #if defined( __STM32__ ) || defined( __KINETIS__ ) || defined( __MCHP__ )
    start_i2c_p                 = I2C_Start_Ptr;
    write_i2c_p                 = I2C_Write_Ptr;
    read_i2c_p                  = I2C_Read_Ptr;
    #elif defined( __TI__ )
    enable_i2c_p                = I2C_Enable_Ptr;
    disable_i2c_p               = I2C_Disable_Ptr;
    set_slave_address_i2c_p     = I2C_Master_Slave_Addr_Set_Ptr;
    write_i2c_p                 = I2C_Write_Ptr;
    read_i2c_p                  = I2C_Read_Ptr;
    #endif
#elif defined( __MIKROC_PRO_FOR_AVR__ )
    #if defined( __LT64__ )
    busy_i2c_p                  = TWI_Busy;
    status_i2c_p                = TWI_Status;
    close_i2c_p                 = TWI_Close;
    start_i2c_p                 = TWI_Start;
    stop_i2c_p                  = TWI_Stop;
    write_i2c_p                 = TWI_Write;
    read_i2c_p                  = TWI_Read;
    #elif defined( __GT64__ )
    busy_i2c_p                  = TWIC_Busy;
    status_i2c_p                = TWIC_Status;
    close_i2c_p                 = TWIC_Close;
    start_i2c_p                 = TWIC_Start;
    stop_i2c_p                  = TWIC_Stop;
    write_i2c_p                 = TWIC_Write;
    read_i2c_p                  = TWIC_Read;
    #endif
#elif defined( __MIKROC_PRO_FOR_PIC__ )
    is_idle_i2c_p               = I2C1_Is_Idle;
    start_i2c_p                 = I2C1_Start;
    stop_i2c_p                  = I2C1_Stop;
    restart_i2c_p               = I2C1_Repeated_Start;
    write_i2c_p                 = I2C1_Wr;
    read_i2c_p                  = I2C1_Rd;
#elif defined( __MIKROC_PRO_FOR_PIC32__ )
    is_idle_i2c_p               = I2C_Is_Idle_Ptr;
    start_i2c_p                 = I2C_Start_Ptr;
    stop_i2c_p                  = I2C_Stop_Ptr;
    restart_i2c_p               = I2C_Restart_Ptr;
    write_i2c_p                 = I2C_Write_Ptr;
    read_i2c_p                  = I2C_Read_Ptr;
#elif defined( __MIKROC_PRO_FOR_DSPIC__ )
    is_idle_i2c_p               = I2C_Is_Idle;
    start_i2c_p                 = I2C_Start;
    stop_i2c_p                  = I2C_Stop;
    restart_i2c_p               = I2C_Restart;
    write_i2c_p                 = I2C_Write;
    read_i2c_p                  = I2C_Read;
#elif defined( __MIKROC_PRO_FOR_8051__ )
    status_i2c_p                = TWI_Status;
    close_i2c_p                 = TWI_Close;
    start_i2c_p                 = TWI_Start;
    stop_i2c_p                  = TWI_Stop;
    write_i2c_p                 = TWI_Write;
    read_i2c_p                  = TWI_Read;
#elif defined( __MIKROC_PRO_FOR_FT90x__ )
    soft_reset_i2c_p            = I2CM_Soft_Reset_Ptr;
    set_slave_address_i2c_p     = I2CM_Set_Slave_Address_Ptr;
    write_i2c_p                 = I2CM_Write_Bytes_Ptr;
    read_i2c_p                  = I2CM_Read_Bytes_Ptr;
#endif
#if defined( __MIKROC_PRO_FOR_ARM__ )   || \
    defined( __MIKROC_PRO_FOR_AVR__ )   || \
    defined( __MIKROC_PRO_FOR_DSPIC__ ) || \
    defined( __MIKROC_PRO_FOR_PIC32__ ) || \
    defined( __MIKROC_PRO_FOR_8051__ )
    write_spi_p                  = SPI_Wr_Ptr;
    read_spi_p                  = SPI_Rd_Ptr;
#elif defined( __MIKROC_PRO_FOR_PIC__ )
    write_spi_p             = SPI1_Write;
    read_spi_p              = SPI1_Read;
#elif defined( __MIKROC_PRO_FOR_FT90x__ )
    write_spi_p             = SPIM_Wr_Ptr;
    read_spi_p              = SPIM_Rd_Ptr;
#endif
#if defined( __MIKROC_PRO_FOR_ARM__ )        || \
    defined( __MIKROC_PRO_FOR_AVR__ )        || \
    defined( __MIKROC_PRO_FOR_DSPIC__ ) || \
    defined( __MIKROC_PRO_FOR_PIC32__ ) || \
    defined( __MIKROC_PRO_FOR_8051__ )  || \
    defined( __MIKROC_PRO_FOR_FT90x__ ) || \
    defined( __MIKROC_PRO_FOR_PIC__ )
    write_uart_p = UART_Wr_Ptr;
#endif
    _millis = 0;
}

void hal_wifi7_tick( void )
{
    _millis++;
}

void hal_wifi_isr( void )
{
    m2m_EintHandler();
}

uint32_t m2mStub_GetOneMsTimer(void)
{
    return _millis;
}

void m2mStub_PinSet_CE(t_m2mWifiPinAction action)
{
    if( action == M2M_WIFI_PIN_LOW )
    {
        WIFI_7_EN = 0;

    }
    else if ( action ==  M2M_WIFI_PIN_HIGH )
    {
        WIFI_7_EN = 1;
    }
}

void m2mStub_PinSet_RESET(t_m2mWifiPinAction action)
{
    if( action == M2M_WIFI_PIN_LOW )
    {
        WIFI_7_RST = 0;

    }
    else if ( action ==  M2M_WIFI_PIN_HIGH )
    {
        WIFI_7_RST = 1;
    }
}

void m2mStub_PinSet_SPI_SS(t_m2mWifiPinAction action)
{
    if( action == M2M_WIFI_PIN_LOW )
    {
        WIFI_7_CS = 0;

    }
    else if ( action ==  M2M_WIFI_PIN_HIGH )
    {
        WIFI_7_CS = 1;
    }
}

void m2mStub_SpiTxRx
(
        uint8_t *p_txBuf,
        uint16_t txLen,
        uint8_t *p_rxBuf,
        uint16_t rxLen
)
{
    uint8_t *in;
    uint8_t *out;
    uint16_t count;

    if( txLen )
        in = p_txBuf;

    if( rxLen )
        out = p_rxBuf;

    count = MAX_RET(txLen, rxLen);

#if defined( __MIKROC_PRO_FOR_ARM__ )   || \
    defined( __MIKROC_PRO_FOR_AVR__ )   || \
    defined( __MIKROC_PRO_FOR_DSPIC__ ) || \
    defined( __MIKROC_PRO_FOR_PIC32__ ) || \
    defined( __MIKROC_PRO_FOR_8051__ )  || \
    defined( __MIKROC_PRO_FOR_FT90x__ ) || \
    defined( __MIKROC_PRO_FOR_PIC__ )
    while( count-- )
    {
        if( txLen && rxLen )
        {
            *out++ = read_spi_p( *in++ );
            txLen--;
            rxLen--;
        }
        else if( txLen && !rxLen )
        {
            write_spi_p( *in++ );
            txLen--;
        }
        else if( !txLen && rxLen )
        {
            *out++ = read_spi_p( 0x00 );
            rxLen--;
        }
    }
#endif
}

void dprintf(char* text, ... )
{
    LOG( text );
}

void printf(char* text )
{
    LOG( text );
}

void strcpycon( uint8_t *dest, const char* src )
{
    uint16_t i = 0;
    
    while( src[i] != 0 )
        *(dest + i) = src[i];
}

uint16_t strlencon( const char* text )
{
    uint16_t i;
    
    while( text[i] != 0 )
        i++;

    return i - 1;
}

/*                                                                 End of File
 ******************************************************************************/
